<?php

namespace Tablesome\Components\Table;

use \Tablesome\Includes\Settings\Tablesome_Getter;

if (!class_exists('\Tablesome\Components\Table\Model')) {
    class Model
    {
        public function get_viewProps($args = [])
        {
            $this->collection = $this->get_collectionProps($args);
            $this->collection = array_merge($this->collection, $args);

            return [
                'collection' => $this->collection,
                'items' => $this->get_itemProps($this->collection),
            ];
        }

        public function get_collectionProps($args = [])
        {
            $table_mode = isset($args['table_mode']) ? $args['table_mode'] : 'read-only';
            $num_of_records_per_page = Tablesome_Getter::get('num_of_records_per_page');
            $page_limit = isset($args['page_limit']) && !empty($args['page_limit']) ? $args['page_limit'] : $num_of_records_per_page;
            $hide_table_header = isset($args['hide_table_header']) ? $args['hide_table_header'] : Tablesome_Getter::get('hide_table_header');
            $sort = isset($args['sorting']) ? $args['sorting'] : Tablesome_Getter::get('sorting');
            $filter = isset($args['filters']) ? $args['filters'] : Tablesome_Getter::get('filters');
            $search = isset($args['search']) ? $args['search'] : Tablesome_Getter::get('search');
            // $table_layout_mode =  isset($args['table_layout_mode']) ? $args['table_layout_mode'] : Tablesome_Getter::get('table_layout_mode');
            $mobile_layout_mode = isset($args['mobile_layout_mode']) ? $args['mobile_layout_mode'] : Tablesome_Getter::get('mobile_layout_mode');
            $custom_style = Tablesome_Getter::get('style_disable') ? true : false;

            return [
                'rowLimit' => TABLESOME_MAX_RECORDS_TO_READ,
                'columnLimit' => TABLESOME_MAX_COLUMNS_TO_READ,
                'defaultColumnName' => __("Column name", 'tablesome'),
                'rowDeleteMessage' => __("Are you sure want's to Delete the selected Row?", 'tablesome'),
                'columnDeleteMessage' => __("Are you sure want's to Delete the selected Column?", 'tablesome'),
                'rowLimitMessage' => __("Couldn't remove the row. The table must have one row at least!", 'tablesome'),
                'columnLimitMessage' => __("Couldn't remove the Column. The table must have one Column at least!", 'tablesome'),

                'mode' => $table_mode,
                'numOfRecordsPerPage' => $page_limit,
                'hideTableHeader' => $hide_table_header,
                'sort' => $sort,
                'filter' => $filter,
                'search' => $search,
                'customStyle' => $custom_style,
                // 'table_layout_mode' => $table_layout_mode,
                'mobile_layout_mode' => $mobile_layout_mode,
                'show_table_footer' => isset($args['show_table_footer']) ? $args['show_table_footer'] : false,
            ];
        }

        public function get_itemProps($collection = [])
        {
            $tablesome_db = new \Tablesome\Includes\Modules\TablesomeDB\TablesomeDB();

            // $table = new \Tablesome\Includes\Core\Table();
            $table_id = isset($collection['post_id']) ? $collection['post_id'] : $collection['table_id'];
            // $table_data = $table->get_table_data($table_id, $collection);
            $table_meta = get_tablesome_data($table_id);
            $columns = isset($table_meta['columns']) && !empty($table_meta['columns']) ? $table_meta['columns'] : [];

            $table_data['columns'] = $this->get_columns($columns, $collection);

            /** Getting Records */
            $table_instance = $tablesome_db->create_table_instance($table_id);
            $number = isset($collection['pagination']) && $collection['pagination'] == 1 ? $collection['page_limit'] : 0;

            $query = $tablesome_db->query(array(
                'table_id' => $table_id,
                'table_name' => $table_instance->name,
                'number' => $number,
                'orderby' => array('rank_order', 'id'),
                'order' => 'asc',
            ));

            $records = isset($query->items) ? $query->items : [];
            $table_data["rows"] = $tablesome_db->get_formatted_rows($records, $table_meta, $collection);

            return $table_data;
        }

        protected function get_rows($table_data)
        {
            $processed_rows = [];
            $rows = isset($table_data['records']) && !empty($table_data['records']) ? $table_data['records'] : [];
            // Create New table
            if (empty($rows)) {
                array_push($processed_rows, [
                    "record_id" => 0,
                    "content" => [""],
                ]);
            }

            $columns = isset($table_data['columns']) && !empty($table_data['columns']) ? $table_data['columns'] : [];

            foreach ($rows as $index => $row) {
                array_push($processed_rows, $this->get_row($row, $columns));
            }

            // unset the $rows after get the row informations
            unset($rows);

            return $processed_rows;
        }

        protected function get_row($row, $columns)
        {

            $cells = isset($row['content']) ? $row['content'] : [];

            if (empty($cells)) {
                return $row;
            }
            /** get exclude column ids */
            $exclude_column_ids = isset($this->collection['exclude_column_ids']) && !empty($this->collection['exclude_column_ids']) ? explode(",", $this->collection['exclude_column_ids']) : [];

            foreach ($cells as $cell_key => $cell_data) {

                /** remove the cell if that cell key (or) ID has found in exclude columns array */
                if (in_array($cell_key, $exclude_column_ids)) {
                    unset($row['content'][$cell_key]);
                    continue;
                }

                // $value = isset($cell_data["value"]) && !empty($cell_data["value"]) ? $cell_data["value"] : $cell_data;
                $cell = [
                    'type' => get_tablesome_cell_type($cell_key, $columns),
                    'html' => isset($cell_data) && isset($cell_data["html"]) && !empty($cell_data["html"]) ? $cell_data["html"] : $cell_data["value"],
                    'value' => isset($cell_data) && isset($cell_data["value"]) ? $cell_data["value"] : "",
                ];

                if (isset($cell_data["link"]) && !empty($cell_data["link"])) {
                    $cell = array_merge($cell, ["link" => $cell_data["link"]]);
                }

                if (isset($cell_data["linkText"]) && !empty($cell_data["linkText"])) {
                    $cell = array_merge($cell, ["linkText" => $cell_data["linkText"]]);
                }

                $cell = apply_filters("tablesome_get_cell_data", $cell);

                $row['content'][$cell_key] = $cell;
            }

            return $row;
        }

        public function get_columns(array $columns, array $collectionProps)
        {
            // $exclude_column_ids values received as string
            $exclude_column_ids = isset($collectionProps['exclude_column_ids']) && !empty($collectionProps['exclude_column_ids']) ? explode(",", $collectionProps['exclude_column_ids']) : [];

            foreach ($columns as $index => $column) {

                $columns[$index]["name"] = html_entity_decode($column["name"]);

                if (!empty($exclude_column_ids) && isset($column['id']) && in_array($column['id'], $exclude_column_ids)) {
                    /** remove the exclude columns */
                    unset($columns[$index]);
                }
            }
            // re-arranging the orders.
            $columns = array_values($columns);

            return $columns;
        }
    }
}