<?php

namespace Tablesome\Includes;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!class_exists('\Tablesome\Includes\Cron')) {
    class Cron
    {

        /** Tablesome cron schedular hook name */
        public $cron_action_hook = 'tablesome/send_data_to_amplitude';

        /**  Schedular processed option name. This value updated when the data sent to amplitude */
        public $cron_processed_option_name = "tablesome_cron_processed";

        public function __construct()
        {
            /** Just for temp soln, the schedular run agin after complete the schedule event task */
            $this->did_cron_processed = get_option($this->cron_processed_option_name);
            $this->fs_utils = new \Tablesome\Includes\Freemius_Utils();

        }

        public function event_setup()
        {
            if ($this->did_cron_processed || !$this->fs_utils->can_track()) {
                return;
            }
            $this->start();
        }

        public function set_intervals($schedules)
        {
            $schedules['tablesome_data_interval'] = array(
                'interval' => 7 * 24 * 60 * 60, // Run the CRON in every week
                'display' => __('Once Weekly', 'tablesome'),
            );
            // $schedules['tablesome_data_test_interval'] = array(
            //     'interval' => 300, // Testing Purpose, Run the CRON in every 5 min
            //     'display' => __('Every 5 min', 'tablesome'),
            // );
            return $schedules;
        }

        public function run(array $args = array())
        {
            if ($this->did_cron_processed || !$this->fs_utils->can_track()) {
                return;
            }

            $controller = new \Tablesome\Includes\Tracking\Controller();
            $controller->send_data();

            /** Clear the scheduled event */
            $this->action('clear');

            /** Update the "tablesome_cron_processed" option is done */
            $this->cron_processed();
        }

        public function cron_processed()
        {
            update_option($this->cron_processed_option_name, 1);
        }

        public function clear()
        {
            /** Clear the CRON sh event */
            wp_clear_scheduled_hook($this->cron_action_hook);
        }

        public function start(array $args = array())
        {
            // Ref: https://www.smashingmagazine.com/2013/10/schedule-events-using-wordpress-cron/
            $timestamp = wp_next_scheduled($this->cron_action_hook);
            if ($timestamp == false) {
                /** Single Event Schedule */

                if (TABLESOME_ENV_MODE == 'production') {
                    $interval = time() + (7 * 24 * 60 * 60);
                } else {
                    $interval = time() + 300;
                }

                wp_schedule_single_event($interval, $this->cron_action_hook, []);

                /*** Recurrence Scheduling  */
                // wp_schedule_event(time(), TABLESOME_SCHEDULE_INTERVAL, $this->cron_action_hook);
            }
        }

        public function action($type, array $args = array())
        {
            if ($type == 'clear') {
                $this->clear();
            } else if ($type == 'start') {
                if (!$this->fs_utils->can_track()) {
                    $this->action('clear');
                    return;
                }
                $this->event_setup();
            }
        }
    }
}