<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!class_exists('\Tablesome')) {
    class Tablesome
    {
        public function __construct()
        {
            $this->setup_autoload();
            $this->load_library();
            $this->load_tablesome_functions();
            $this->register_cpt_and_taxonomy();
            $this->load_update_handler();
            $this->load_actions();
            $this->load_filters();
            $this->load_shorcodes();

            $this->load_notification();

            $this->load_features();

        }

        protected function setup_autoload()
        {
            require_once TABLESOME_PATH . '/vendor/autoload.php';
        }

        protected function load_library()
        {
            if (!class_exists("\Pauple\Pluginator\Library")) {
                wp_die("\"freemius/wordpress-sdk\" and \"Codestar Framework\" library was not installed, \"Tablesome\" is depend on it. Do run \"composer update\".");
            }

            $library = new \Pauple\Pluginator\Library();
            $library::register_libraries(['codestar', 'freemius']);
        }

        public function load_tablesome_functions()
        {
            require_once TABLESOME_PATH . 'includes/functions.php';
            require_once TABLESOME_PATH . 'includes/settings/getter.php';
        }

        /*  Register Tablesome Post types and its Taxonomies */
        public function register_cpt_and_taxonomy()
        {
            $cpt = new \Tablesome\Includes\Cpt();
            $cpt->register();
        }

        public function load_actions()
        {
            new \Tablesome\Includes\Actions();
        }

        public function load_filters()
        {
            new \Tablesome\Includes\Filters();
        }

        public function load_update_handler()
        {
            $upgrade = new \Tablesome\Includes\Update\Upgrade();
            $upgrade::init();
        }

        /*  Tablesome Shortcode */
        public function load_shorcodes()
        {
            new \Tablesome\Includes\Shortcodes();

            /** Init shortcode builder  */
            $builder = new \Tablesome\Includes\Shortcode_Builder\Builder();
            $builder->init();
        }

        public function load_notification()
        {
            if (is_admin()) {

                $message = '<p><b>' . __("Congrats! you have created the 1st table with Tablesome,", "tablesome") . '</b> ' . __("that's awesome!", "tablesome");

                $review_notice_args = array(
                    'post_type' => TABLESOME_CPT,
                    'domain' => TABLESOME_DOMAIN,

                    /** users review url. */
                    'plugin_review_url' => 'https://wordpress.org/support/plugin/tablesome/reviews/#new-post',

                    /**
                     * This is the limit to show the review notification when users clicked the maybe later option.
                     * Will show the review notification if the post interval limit is met again.
                     */
                    'post_interval_limit' => 3,

                    /*** This is the limit to show the review notification when the total no of posts is met with that limit   .  **/
                    'post_limit' => 3,

                    /** Review notification message */
                    'message' => $message,

                    /*** show this notification message if the interval limit is med  */
                    'interval_message' => '<p><b>' . __("Congrats!", "tablesome") . ' %d+' . __("Tablesome tables are Created: ", "tablesome") . '</b>' . __("Hey,  I noticed you have created more than", "tablesome") . ' %d ' . __("tables using Tablesome - that's awesome! ", 'tablesome') . '</p>',
                );

                $review_notice = new \Pauple\Pluginator\ReviewNotification();
                $review_notice->init($review_notice_args);
            }
        }

        public function load_features()
        {
            $post_id = isset($_GET['post']) ? $_GET['post'] : 0;
            if (empty($post_id)) {return;}
            $post = get_post($post_id);

            if (empty($post) || $post->post_type != 'tablesome_cpt') {return;}

            if (pauple_is_feature_active('tablesomDB')) {
                $tablesomeDB = new \Tablesome\Includes\Modules\TablesomeDB\TablesomeDB();
                $tablesomeDB->create_table_instance($post->ID);
            }
        }
    }
    new Tablesome();
}