<?php

namespace Tablesome\Includes\Update;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!class_exists('\Tablesome\Includes\Update\Upgrade_List')) {
    class Upgrade_List
    {

        public function __construct()
        {
            $this->crud = new \Tablesome\Includes\Db\CRUD();
            $this->helpers = new \Tablesome\Includes\Helpers();
        }

        public function get_upgrades()
        {
            $upgrades = [
                '0.5.9.1' => 'upgrade_v0591',
                '0.5.8' => 'upgrade_v058',
                '0.4.1' => 'upgrade_v041',
                '0.4' => 'upgrade_v040',
                '0.2.6' => 'upgrade_v026',
                '0.0.2' => 'upgrade_v002',
                '0.2' => 'upgrade_v02',
            ];

            return $upgrades;
        }

        public function upgrade_v0591()
        {
            $upgrade_v0591_done = true;
            $tablesome_version = get_option("tablesome_version");
            error_log('[$tablesome_version] : ' . $tablesome_version);
            if ($tablesome_version === "0.5.9.1") {
                error_log('[Migration already processed]');
                return $upgrade_v0591_done;
            }
            error_log('[Start Migration]');

            $migrate_old_data = new \Tablesome\Includes\Modules\Migrate_Old_Crud_Data_To_TablesomeDB();
            $migrate_old_data->run();
            return $upgrade_v0591_done;
        }

        public function upgrade_v058()
        {
            $upgrade_v058_done = true;
            $tablesome_version = get_option("tablesome_version");
            if ($tablesome_version === "0.5.8") {
                return $upgrade_v058_done;
            }

            global $wpdb;
            $table_name = $wpdb->prefix . TABLESOME_RECORDS_TABLE_NAME;

            $upgrade_v058_done = $wpdb->query("UPDATE $table_name SET `rank_order` = '0|0zzzzs:' WHERE `rank_order` = '0|000000:'");

            return $upgrade_v058_done;
        }

        public function upgrade_v041()
        {
            $upgrade_v41_done = true;
            $tablesome_version = get_option("tablesome_version");
            if ($tablesome_version === "0.4.1") {
                return $upgrade_v41_done;
            }

            /** Use this option to show the Tablesome opt-in notice to the admin pages.( < v0.4.1)  */
            update_option("tablesome_opt_in_notices", 1);
            update_option("tablesome_can_track_events", 'disabled');

            return $upgrade_v41_done;
        }

        public function upgrade_v040()
        {
            $upgrade_v40_done = true;
            $tablesome_version = get_option("tablesome_version");
            if ($tablesome_version === "0.4") {
                return $upgrade_v40_done;
            }

            global $wpdb;
            $table_name = $wpdb->prefix . TABLESOME_RECORDS_TABLE_NAME;
            $column_name = "rank_order";
            $column_query = "ALTER TABLE $table_name ADD $column_name varchar(255) NOT NULL DEFAULT ''";
            maybe_add_column($table_name, $column_name, $column_query);

            return $upgrade_v40_done;
        }

        public function upgrade_v026()
        {
            $upgrade_v026_done = true;
            $tablesome_version = get_option("tablesome_version");
            if ($tablesome_version === "0.2.6") {
                return $upgrade_v026_done;
            }

            $tables = get_posts([
                'numberposts' => -1,
                'post_type' => TABLESOME_CPT,
            ]);

            if (!isset($tables) || empty($tables)) {
                return $upgrade_v026_done;
            }

            foreach ($tables as $table) {
                $this->run_migration_v026($table->ID);
            }

            return $upgrade_v026_done;
        }

        public function upgrade_v002()
        {
            $upgrade_v002_done = true;
            $tablesome_version = get_option("tablesome_version");
            $tablesome_upgrades = get_option("tablesome_upgrades");
            if ($tablesome_version === "0.0.2" || !empty($tablesome_upgrades)) {
                return $upgrade_v002_done;
            }

            // error_log('!!! upgrade_v002 in processing !!!');
            $tables = get_posts([
                'numberposts' => -1,
                'post_type' => TABLESOME_CPT,
            ]);

            if (!isset($tables) || empty($tables)) {
                // error_log('!!! upgrade_v002 done !!!');
                return $upgrade_v002_done;
            }

            foreach ($tables as $table) {
                $old_table_data = get_post_meta($table->ID, 'tablesome_data', true);
                $new_table_data = $this->run_migration_v002($old_table_data);
                update_post_meta($table->ID, 'tablesome_data', $new_table_data);
            }
            // error_log('!!! upgrade_v002 done !!!');
            return $upgrade_v002_done;
        }

        public function upgrade_v02()
        {
            $upgrade_v02_done = true;
            $tablesome_version = get_option("tablesome_version");
            $tablesome_upgrades = get_option("tablesome_upgrades");
            if ($tablesome_version === "0.2") {
                return $upgrade_v02_done;
            }

            $tables = get_posts([
                'numberposts' => -1,
                'post_type' => TABLESOME_CPT,
            ]);

            $this->run_tablesome_records_table();
            foreach ($tables as $table) {
                $old_table_data = get_post_meta($table->ID, 'tablesome_data', true);
                $new_table_data = $this->run_migration_v02($table->ID, $old_table_data);
                update_post_meta($table->ID, 'tablesome_data', $new_table_data);
            }
            return $upgrade_v02_done;
        }

        public function run_migration_v026($table_id)
        {
            $records = $this->crud->get_all_rows($table_id);
            $records = $this->helpers->get_decoded_rows($records);

            foreach ($records as $record) {
                if (isset($record["content"]) && !empty($record["content"])) {
                    foreach ($record["content"] as $column_id => $cell) {
                        if (!is_array($cell)) {
                            $record["content"][$column_id] = ["value" => $cell];
                        }
                    }
                    $this->crud->update($table_id, $record["record_id"], $record["content"]);
                }
            }

            return true;
        }

        public function run_migration_v002($data)
        {
            $new_data = [
                'options' => [],
                'columns' => [],
                'rows' => [],
                'meta' => [
                    'last_column_id' => 0,
                ],
            ];

            if (!isset($data['columns']) || empty($data['columns']) || !isset($data['rows']) || empty($data['rows'])) {
                return $new_data;
            }

            $column_id = 0;
            foreach ($data['columns'] as $column_name) {
                $column_id++;
                $new_data['columns'][] = [
                    'id' => $column_id,
                    'name' => $column_name,
                    'format' => 'text',
                ];
            }

            foreach ($data['rows'] as $row) {
                $column_id = 0;
                $new_row = [];
                foreach ($row as $cell_key => $cell_value) {
                    $column_id++;
                    $new_row[$column_id] = $cell_value;
                }
                array_push($new_data['rows'], $new_row);
            }

            $new_data['meta']['last_column_id'] = $column_id;

            return $new_data;
        }

        public function run_tablesome_records_table()
        {
            $table = new \Tablesome\Includes\Db\Tablesome_Table();
            $table->create();
        }

        public function run_migration_v02($table_id, $old_table_data)
        {
            $old_rows = isset($old_table_data['rows']) ? $old_table_data['rows'] : [];
            if (empty($old_rows)) {
                return $old_table_data;
            }
            $table = new \Tablesome\Includes\Core\Table();
            foreach ($old_rows as $row) {
                $insert_row = $table->insert_row(
                    array(
                        'content' => $row,
                        'post_id' => $table_id,
                    )
                );
            }
            $old_table_data['rows'] = [];
            return $old_table_data;
        }

    } // END CLASS
}